package app

import (
	"emperror.dev/errors"
	"git.sleepycat.moe/sam/mercury/config"
	"git.sleepycat.moe/sam/mercury/internal/database/sql"
	"git.sleepycat.moe/sam/mercury/web/templates"
	"github.com/flosch/pongo2/v6"
	"github.com/go-chi/chi/v5"
	"github.com/go-chi/chi/v5/middleware"
)

type App struct {
	Router chi.Router

	Config   config.Config
	Database *sql.Base

	tmpl *pongo2.TemplateSet
}

func NewApp(cfg config.Config, db *sql.Base) (*App, error) {
	app := &App{
		Router:   chi.NewRouter(),
		Config:   cfg,
		Database: db,
	}

	tmpl, err := templates.New(cfg.Core.Dev)
	if err != nil {
		return nil, errors.Wrap(err, "creating templates")
	}
	app.tmpl = tmpl

	app.Router.Use(app.Logger)
	app.Router.Use(middleware.Recoverer)

	return app, nil
}

func (a *App) Account(q ...sql.Querier) *sql.AccountStore {
	if len(q) == 0 || q[0] == nil {
		return sql.NewAccountStore(a.Database.PoolQuerier())
	}
	return sql.NewAccountStore(q[0])
}

func (a *App) Blog(q ...sql.Querier) *sql.BlogStore {
	if len(q) == 0 || q[0] == nil {
		return sql.NewBlogStore(a.Database.PoolQuerier())
	}
	return sql.NewBlogStore(q[0])
}

func (a *App) Post(q ...sql.Querier) *sql.PostStore {
	if len(q) == 0 || q[0] == nil {
		return sql.NewPostStore(a.Database.PoolQuerier())
	}
	return sql.NewPostStore(q[0])
}
